@@ -11,7 +11,7 @@ use HTTP::Request();
 use HTTP::Response();
 use POSIX();
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 sub _CHUNK_SIZE                { return 4096 }
 sub _FILE_READ_SIZE            { return 8192 }
@@ -150,6 +150,10 @@ sub _debug {
         else {
             $self->{_debug_buffer} = $string;
         }
+        while ( $self->{_debug_buffer} =~ s/\A([^\n]+\r?\n)//smx ) {
+            print {*STDERR} "$direction$1"
+              or Carp::croak("Failed to write to STDERR:$EXTENDED_OS_ERROR");
+        }
         $self->{_previous_direction} = $direction;
     }
     return;
@@ -217,6 +221,12 @@ sub _connect {
     return $self->{_socket};
 }
 
+sub _disconnect {
+    my ($self) = @_;
+    delete $self->{_socket};
+    return;
+}
+
 sub _process_icap_headers {
     my ( $self, $icap_headers, $icap_method ) = @_;
     my $quoted_pair   = qr/\\./smx;
@@ -266,6 +276,10 @@ sub _get_icap_header {
           or Carp::croak("Failed to read from $icap_uri:$EXTENDED_OS_ERROR");
         $peek_buffer .= $buffer;
     }
+    if ( $peek_buffer =~ /^ICAP\/1[.]0[ ]([45]\d\d)[ ]/smx ) {
+        $self->_disconnect();
+        Carp::croak("ICAP Server returned a $1 error");
+    }
     return $peek_buffer;
 }
 
@@ -317,12 +331,7 @@ qr/\r?\nEncapsulated:[ ]?(?:re[sq]\-hdr=(\d+),[ ]?)?(req|res|null)\-body=(\d+)(?
         }
         elsif ( $icap_headers =~ /^ICAP\/1[.]0[ ]204[ ]/smx ) {
             $self->_process_icap_headers( $icap_headers, $params{icap_method} );
-            if ( defined $params{content_handle} ) {
-                seek $params{content_handle}, Fcntl::SEEK_SET(), 0
-                  or Carp::croak(
-"Failed to seek to start of content handle:$EXTENDED_OS_ERROR"
-                  );
-            }
+            $self->_reset_content_handle( $params{content_handle} );
             $self->_debug_flush();
             if ( defined $params{response} ) {
                 return ( $params{response}, $params{content_handle} );
@@ -554,12 +563,11 @@ sub response {
         if ( $peek_buffer =~ /^ICAP\/1[.]0[ ]100[ ]/smx ) {
         }
         elsif ( $peek_buffer =~ /^ICAP\/1[.]0[ ]204[ ]/smx ) {
-            seek $content_handle, Fcntl::SEEK_SET(), 0
-              or Carp::croak(
-                "Failed to seek to start of content handle:$EXTENDED_OS_ERROR");
+            $self->_reset_content_handle($content_handle);
             return ( $response, $content_handle );
         }
         elsif ( $peek_buffer =~ /^ICAP\/1[.]0[ ]([45]\d\d)[ ]/smx ) {
+            $self->_disconnect();
             Carp::croak("ICAP Server returned a $1 error");
         }
         else {
@@ -598,6 +606,16 @@ sub response {
     );
 }
 
+sub _reset_content_handle {
+    my ( $self, $content_handle ) = @_;
+    if ( defined $content_handle ) {
+        seek $content_handle, Fcntl::SEEK_SET(), 0
+          or Carp::croak(
+            "Failed to seek to start of content handle:$EXTENDED_OS_ERROR");
+    }
+    return;
+}
+
 sub request {
     my ( $self, $request, $content_handle ) = @_;
     $self->_connect();
@@ -651,12 +669,11 @@ sub request {
         if ( $peek_buffer =~ /^ICAP\/1[.]0[ ]100[ ]/smx ) {
         }
         elsif ( $peek_buffer =~ /^ICAP\/1[.]0[ ]204[ ]/smx ) {
-            seek $content_handle, Fcntl::SEEK_SET(), 0
-              or Carp::croak(
-                "Failed to seek to start of content handle:$EXTENDED_OS_ERROR");
+            $self->_reset_content_handle($content_handle);
             return ( $request, $content_handle );
         }
         elsif ( $peek_buffer =~ /^ICAP\/1[.]0[ ]([45]\d\d)[ ]/smx ) {
+            $self->_disconnect();
             Carp::croak("ICAP Server returned a $1 error");
         }
         else {
@@ -708,14 +725,14 @@ Net::ICAP::Client - A client implementation of the ICAP (RFC 3507) protocol
 
 =head1 VERSION
 
-Version 0.03
+Version 0.04
 
 =head1 SYNOPSIS
 
     use Net::ICAP::Client;
 
     my $icap = Net::ICAP::Client->new('icap://icap-proxy.example.com/');
-    my $request = HTTP::Request->new( 'POST', 'https://www.example.com/path', $request_headers );
+    my $request = HTTP::Request->new( 'POST' => 'https://www.example.com/path' );
     my ( $headers, $body ) = $icap->request( $request );
     if ($headers->isa('HTTP::Request') {
 	# forward request to intended destination
@@ -784,7 +801,7 @@ $icap->uri() returns the current URI of the remote ICAP server as a L<URI|URI> o
 
     my $icap = Net::ICAP::Client->new('icap://icap-proxy.example.com/');
     my $request_headers = HTTP::Headers->new();
-    my $request = HTTP::Request->new( 'POST', "https://www.example.com/path?name=value", $request_headers, "name2=value2" );
+    my $request = HTTP::Request->new( 'POST' => "https://www.example.com/path?name=value", $request_headers, "name2=value2" );
     my ( $request_or_response_headers, $filehandle_containing_possibly_updated_body ) = $icap->request( $request, $filehandle_containing_request_body );
 
 $icap->request() expects an L<HTTP::Request|HTTP::Request> object and an optional filehandle.  It will return an L<HTTP::Request|HTTP::Request> or an L<HTTP::Response|HTTP::Response> object containing the request or response without the body and a filehandle containing the body.
@@ -835,7 +852,7 @@ Failed to do a disk operation.  Check disk settings for the mount point belongin
  
 =item C<< ICAP Server returned a %s error >>
  
-The remote ICAP server returned an error.  Capture the network traffic and enter a bug report
+The remote ICAP server returned an error.  The TCP connection to the remote ICAP server will be automatically disconnected.  Capture the network traffic and enter a bug report
  
 =item C<< Failed to parse chunking length >>
  
@@ -863,11 +880,11 @@ Net::ICAP::Client requires no configuration files or environment variables.
 
 Net::ICAP::Client requires the following non-core modules
  
+  HTTP::Request
+  HTTP::Response
   IO::Socket::INET
   IO::Socket::SSL
   URI
-  HTTP::Request
-  HTTP::Response
 
 =head1 INCOMPATIBILITIES
 
